@@ -142,7 +142,8 @@ des_meta = {
     'filterset': 'des',
     'retrieved': '22 March 2013',
     'description': 'Dark Energy Camera grizy filter set at airmass 1.3'}
-for name, fname in [('desg', 'bandpasses/des/des_g.dat'),
+for name, fname in [('desu', 'bandpasses/des/des_u.dat'),
+                    ('desg', 'bandpasses/des/des_g.dat'),
                     ('desr', 'bandpasses/des/des_r.dat'),
                     ('desi', 'bandpasses/des/des_i.dat'),
                     ('desz', 'bandpasses/des/des_z.dat'),
@@ -529,6 +530,20 @@ for filt in ['Red']:
                                 args=(relpath,), meta=tess_meta)
 
 
+# GOTO
+goto_meta = {
+    'filterset': 'goto',
+    'retrieved': '16 June 2023',
+    'dataurl': ('http://svo2.cab.inta-csic.es/svo/theory/fps/getdata.php?'
+                'format=ascii&id=Misc'),
+    'description': ('GOTO filters from SVO (includes filter, optics,'
+                    'detector and atmosphere.)')}
+for filt in ['B', 'G', 'L', 'R']:
+    name = 'goto' + filt[0].lower()
+    relpath = 'bandpasses/goto/goto.{}'.format(filt)
+    _BANDPASSES.register_loader(name, load_bandpass_remote_aa,
+                                args=(relpath,), meta=goto_meta)
+
 # =============================================================================
 # interpolators
 
