@@ -1,37 +1,50 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Functions to plot light curve data and models."""
-from __future__ import division
-
-import math
 
 import numpy as np
-from astropy.utils.misc import isiterable
-from matplotlib import cm
-from matplotlib import pyplot as plt
-from matplotlib.ticker import MaxNLocator, NullFormatter
-from mpl_toolkits.axes_grid1 import make_axes_locatable
 
-from .models import Source, Model, get_source
-from .spectral import get_bandpass, get_magsystem
-from .photdata import standardize_data, normalize_data
+from .bandpasses import get_bandpass
+from .magsystems import get_magsystem
+from .models import Model
+from .photdata import photometric_data
 from .utils import format_value
 
-__all__ = ['plot_lc', 'animate_source', 'animate_model']
+__all__ = ['plot_lc']
 
 _model_ls = ['-', '--', ':', '-.']
-_cmap = cm.get_cmap('jet_r')
-_cmap_wavelims = (3000., 10000.)
 
-def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
-            xfigsize=None, yfigsize=None, figtext=None, model_label=None,
-            errors=None, ncol=2, figtextsize=1., show_model_params=True,
-            fname=None, **kwargs):
+
+def _add_errorbar(ax, x, y, yerr, filled, markersize=None, color=None):
+    """Add an errorbar to Axes `ax`, allowing an array of markers."""
+    ax.errorbar(x[filled], y[filled], yerr[filled], ls='None',
+                marker='o', markersize=markersize, color=color)
+    notfilled = ~filled
+    ax.errorbar(x[notfilled], y[notfilled], yerr[notfilled], ls='None',
+                mfc='None', marker='o', markersize=markersize, color=color)
+
+
+def _add_plot(ax, x, y, filled, markersize=None, color=None):
+    ax.plot(x[filled], y[filled], marker='o',
+            markersize=markersize, color=color, ls='None')
+    notfilled = ~filled
+    ax.plot(x[notfilled], y[notfilled], marker='o', mfc='None',
+            markersize=markersize, color=color, ls='None')
+
+
+def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab',
+            pulls=True, xfigsize=None, yfigsize=None, figtext=None,
+            model_label=None, errors=None, ncol=2, figtextsize=1.,
+            show_model_params=True, tighten_ylim=False, color=None,
+            cmap=None, cmap_lims=(3000., 10000.), fill_data_marker=None,
+            fname=None, fill_percentiles=None, **kwargs):
     """Plot light curve data or model light curves.
 
     Parameters
     ----------
-    data : astropy `~astropy.table.Table` or similar
-        Table of photometric data points.
+    data : astropy `~astropy.table.Table` or similar, optional
+        Table of photometric data. Must include certain column names.
+        See the "Photometric Data" section of the documentation for required
+        columns.
     model : `~sncosmo.Model` or list thereof, optional
         If given, model light curve is plotted. If a string, the corresponding
         model is fetched from the registry. If a list or tuple of
@@ -46,12 +59,16 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     bands : list, optional
         List of Bandpasses, or names thereof, to plot.
     zp : float, optional
-        Zeropoint to normalize the flux. Default is 25.
+        Zeropoint to normalize the flux in the plot (for the purpose of
+        plotting all observations on a common flux scale). Default is 25.
     zpsys : str, optional
-        Zeropoint system for ``zp``. Default is ``'ab'``.
+        Zeropoint system to normalize the flux in the plot (for the purpose of
+        plotting all observations on a common flux scale).
+        Default is ``'ab'``.
     pulls : bool, optional
-        If True (and if model and data are given), plot pulls. Default is
-        ``True``.
+        If True (and if model and data are given), plot pulls. Pulls are the
+        deviation of the data from the model divided by the data uncertainty.
+        Default is ``True``.
     figtext : str, optional
         Text to add to top of figure. If a list of strings, each item is
         placed in a separate "column". Use newline separators for multiple
@@ -69,8 +86,33 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
         are added to ``figtext`` by default (as two additional columns) so
         that they are printed at the top of the figure. Set this to False to
         disable this behavior.
+    tighten_ylim : bool, optional
+        If true, tighten the y limits so that the model is visible (if any
+        models are plotted).
+    color : str or mpl_color, optional
+        Color of data and model lines in each band. Can be any type of color
+        that matplotlib understands. If None (default) a colormap will be used
+        to choose a color for each band according to its central wavelength.
+    cmap : Colormap, optional
+        A matplotlib colormap to use, if color is None. If both color
+        and cmap are None, a default colormap will be used.
+    cmap_lims : (float, float), optional
+        The wavelength limits for the colormap, in Angstroms. Default is
+        (3000., 10000.), meaning that a bandpass with a central wavelength of
+        3000 Angstroms will be assigned a color at the low end of the colormap
+        and a bandpass with a central wavelength of 10000 will be assigned a
+        color at the high end of the colormap.
+    fill_data_marker : array_like, optional
+        Array of booleans indicating whether to plot a filled or unfilled
+        marker for each data point. Default is all filled markers.
     fname : str, optional
-        Filename to pass to savefig. If `None` (default), figure is returned.
+        Filename to pass to savefig. If None (default), figure is returned.
+    fill_percentiles : (float, float, float), optional
+        When multiple models are given, the percentiles for a light
+        curve confidence interval. The upper and lower perceniles
+        define a fill between region, and the middle percentile
+        defines a line that will be plotted over the fill between
+        region.
     kwargs : optional
         Any additional keyword args are passed to `~matplotlib.pyplot.savefig`.
         Popular options include ``dpi``, ``format``, ``transparent``. See
@@ -81,7 +123,7 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     fig : matplotlib `~matplotlib.figure.Figure`
         Only returned if `fname` is `None`. Display to screen with
         ``plt.show()`` or save with ``fig.savefig(filename)``. When creating
-        many figures, be sure to close with ``plt.close(fig)``. 
+        many figures, be sure to close with ``plt.close(fig)``.
 
     Examples
     --------
@@ -96,34 +138,39 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     Plot the data, displaying to the screen:
 
     >>> fig = plot_lc(data)
-    >>> plt.show()  # doctest: +SKIP
+    >>> plt.show()
 
     Plot a model along with the data:
-    
-    >>> model = sncosmo.Model('salt2')                # doctest: +SKIP
-    >>> model.set(z=0.5, c=0.2, t0=55100., x0=1.547e-5)  # doctest: +SKIP
-    >>> sncosmo.plot_lc(data, model=model)               # doctest: +SKIP
+
+    >>> model = sncosmo.Model('salt2')
+    >>> model.set(z=0.5, c=0.2, t0=55100., x0=1.547e-5)
+    >>> sncosmo.plot_lc(data, model=model)
 
     .. image:: /pyplots/plotlc_example.png
 
     Plot just the model, for selected bands:
 
-    >>> sncosmo.plot_lc(model=model,                     # doctest: +SKIP
-    ...                 bands=['sdssg', 'sdssr'])        # doctest: +SKIP
+    >>> sncosmo.plot_lc(model=model,
+    ...                 bands=['sdssg', 'sdssr'])
 
     Plot figures on a multipage pdf:
 
-    >>> from matplotlib.backends.backend_pdf import PdfPages  # doctest: +SKIP
-    >>> pp = PdfPages('output.pdf')                           # doctest: +SKIP
-    ...
+    >>> from matplotlib.backends.backend_pdf import PdfPages
+    >>> pp = PdfPages('output.pdf')
+
     >>> # Do the following as many times as you like:
-    >>> sncosmo.plot_lc(data, fname=pp, format='pdf')    # doctest: +SKIP
-    ...
+    >>> sncosmo.plot_lc(data, fname=pp, format='pdf')
+
     >>> # Don't forget to close at the end:
-    >>> pp.close()                                       # doctest: +SKIP
+    >>> pp.close()
 
     """
 
+    from matplotlib import pyplot as plt
+    from matplotlib import cm
+    from matplotlib.ticker import MaxNLocator
+    from mpl_toolkits.axes_grid1 import make_axes_locatable
+
     if data is None and model is None:
         raise ValueError('must specify at least one of: data, model')
     if data is None and bands is None:
@@ -142,7 +189,7 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     # Get the model labels
     if model_label is None:
         model_labels = [None] * len(models)
-    elif isinstance(model_label, basestring):
+    elif isinstance(model_label, str):
         model_labels = [model_label]
     else:
         model_labels = model_label
@@ -150,25 +197,49 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
         raise ValueError('if given, length of model_label must match '
                          'that of model')
 
+    # Color options.
+    if color is None:
+        if cmap is None:
+            cmap = cm.get_cmap('jet_r')
+
     # Standardize and normalize data.
     if data is not None:
-        data = standardize_data(data)
-        data = normalize_data(data, zp=zp, zpsys=zpsys)
+        data = photometric_data(data)
+        data = data.normalized(zp=zp, zpsys=zpsys)
+        if not np.all(np.ediff1d(data.time) >= 0.0):
+            sortidx = np.argsort(data.time)
+            data = data[sortidx]
+        else:
+            sortidx = None
 
     # Bands to plot
     if data is None:
         bands = set(bands)
     elif bands is None:
-        bands = set(data['band'])
+        bands = set(data.band)
     else:
-        bands = set(data['band']) & set(bands)
+        bands = set(data.band) & set(bands)
+
+    # ensure bands is a list of Bandpass objects
+    bands = [get_bandpass(b) for b in bands]
+
+    # filled: used only if data is not None. Guarantee array of booleans
+    if data is not None:
+        if fill_data_marker is None:
+            fill_data_marker = np.ones(data.time.shape, dtype=bool)
+        else:
+            fill_data_marker = np.asarray(fill_data_marker)
+            if fill_data_marker.shape != data.time.shape:
+                raise ValueError("fill_data_marker shape does not match data")
+        if sortidx is not None:  # sort like we sorted the data
+            fill_data_marker = fill_data_marker[sortidx]
 
     # Build figtext (including model parameters, if there is exactly 1 model).
     if errors is None:
         errors = {}
     if figtext is None:
         figtext = []
-    elif isinstance(figtext, basestring):
+    elif isinstance(figtext, str):
         figtext = [figtext]
     if len(models) == 1 and show_model_params:
         model = models[0]
@@ -213,7 +284,6 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
 
     # Create the figure and axes.
     fig, axes = plt.subplots(nrow, ncol, figsize=figsize, squeeze=False)
-    
 
     fig.subplots_adjust(left=lspace / figsize[0],
                         bottom=bspace / figsize[1],
@@ -240,8 +310,8 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     # Global min and max of time axis.
     tmin, tmax = [], []
     if data is not None:
-        tmin.append(np.min(data['time']) - 10.)
-        tmax.append(np.max(data['time']) + 10.)
+        tmin.append(np.min(data.time) - 10.)
+        tmax.append(np.max(data.time) + 10.)
     for model in models:
         tmin.append(model.mintime())
         tmax.append(model.maxtime())
@@ -249,13 +319,10 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
     tmax = max(tmax)
     tgrid = np.linspace(tmin, tmax, int(tmax - tmin) + 1)
 
-    # Add 
-
     # Loop over bands
-    bands = list(bands)
-    waves = [get_bandpass(b).wave_eff for b in bands]
+    waves = [b.wave_eff for b in bands]
     waves_and_bands = sorted(zip(waves, bands))
-    for axnum in xrange(ncol * nrow):
+    for axnum in range(ncol * nrow):
         row = axnum // ncol
         col = axnum % ncol
         ax = axes[row, col]
@@ -269,38 +336,56 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
 
         bandname_coords = (0.92, 0.92)
         bandname_ha = 'right'
-        color = _cmap((_cmap_wavelims[1] - wave) /
-                      (_cmap_wavelims[1] - _cmap_wavelims[0]))
+        if color is None:
+            bandcolor = cmap((cmap_lims[1] - wave) /
+                             (cmap_lims[1] - cmap_lims[0]))
+        else:
+            bandcolor = color
 
         # Plot data if there are any.
         if data is not None:
-            mask = data['band'] == band
-            time = data['time'][mask]
-            flux = data['flux'][mask]
-            fluxerr = data['fluxerr'][mask]
-            ax.errorbar(time - toff, flux, fluxerr, ls='None',
-                        color=color, marker='.', markersize=3.)
+            mask = data.band == band
+            time = data.time[mask]
+            flux = data.flux[mask]
+            fluxerr = data.fluxerr[mask]
+            bandfilled = fill_data_marker[mask]
+            _add_errorbar(ax, time - toff, flux, fluxerr, bandfilled,
+                          color=bandcolor, markersize=3.)
 
         # Plot model(s) if there are any.
         lines = []
         labels = []
         mflux_ranges = []
+        mfluxes = []
+        plotci = len(models) > 1 and fill_percentiles is not None
+
         for i, model in enumerate(models):
             if model.bandoverlap(band):
                 mflux = model.bandflux(band, tgrid, zp=zp, zpsys=zpsys)
-                mflux_ranges.append((mflux.min(), mflux.max()))
-                l, = ax.plot(tgrid - toff, mflux,
-                             ls=_model_ls[i%len(_model_ls)],
-                             marker='None', color=color)
-                lines.append(l)
+                if not plotci:
+                    mflux_ranges.append((mflux.min(), mflux.max()))
+                    l, = ax.plot(tgrid - toff, mflux,
+                                 ls=_model_ls[i % len(_model_ls)],
+                                 marker='None', color=bandcolor)
+                    lines.append(l)
+                else:
+                    mfluxes.append(mflux)
             else:
                 # Add a dummy line so the legend displays all models in the
                 # first panel.
                 lines.append(plt.Line2D([0, 1], [0, 1],
-                                        ls=_model_ls[i%len(_model_ls)],
-                                        marker='None', color=color))
+                                        ls=_model_ls[i % len(_model_ls)],
+                                        marker='None', color=bandcolor))
             labels.append(model_labels[i])
 
+        if plotci:
+            lo, med, up = np.percentile(mfluxes, fill_percentiles, axis=0)
+            l, = ax.plot(tgrid - toff, med, marker='None',
+                         color=bandcolor)
+            lines.append(l)
+            ax.fill_between(tgrid - toff, lo, up, color=bandcolor,
+                            alpha=0.4)
+
         # Add a legend, if this is the first axes and there are two
         # or more models to distinguish between.
         if row == 0 and col == 0 and model_label is not None:
@@ -310,7 +395,8 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
             bandname_ha = 'left'
 
         # Band name in corner
-        ax.text(bandname_coords[0], bandname_coords[1], band,
+        text = band.name if band.name is not None else str(band)
+        ax.text(bandname_coords[0], bandname_coords[1], text,
                 color='k', ha=bandname_ha, va='top', transform=ax.transAxes)
 
         ax.axhline(y=0., ls='--', c='k')  # horizontal line at flux = 0.
@@ -318,7 +404,7 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
 
         # If we plotted any models, narrow axes limits so that the model
         # is visible.
-        if len(mflux_ranges) > 0:
+        if tighten_ylim and len(mflux_ranges) > 0:
             mfluxmin = min([r[0] for r in mflux_ranges])
             mfluxmax = max([r[1] for r in mflux_ranges])
             ymin, ymax = ax.get_ylim()
@@ -337,14 +423,14 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
         # steal part of the axes and plot pulls
         if show_pulls:
             divider = make_axes_locatable(ax)
-            axpulls = divider.append_axes('bottom', size='30%' , pad=0.15,
+            axpulls = divider.append_axes('bottom', size='30%', pad=0.15,
                                           sharex=ax)
-            mflux = models[0].bandflux(band, time, zp=zp, zpsys=zpsys) 
+            mflux = models[0].bandflux(band, time, zp=zp, zpsys=zpsys)
             fluxpulls = (flux - mflux) / fluxerr
             axpulls.axhspan(ymin=-1., ymax=1., color='0.95')
-            axpulls.axhline(y=0., color=color)
-            axpulls.plot(time - toff, fluxpulls, marker='.',
-                         markersize=5., color=color, ls='None')
+            axpulls.axhline(y=0., color=bandcolor)
+            _add_plot(axpulls, time - toff, fluxpulls, bandfilled,
+                      markersize=4., color=bandcolor)
 
             # Ensure y range is centered at 0.
             ymin, ymax = axpulls.get_ylim()
@@ -373,15 +459,13 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
         else:
             bottomax = ax
 
-        # If this axes is one of the last `ncol`, set x label and rotate
-        # tick labels. Otherwise don't show tick labels.
+        # If this axes is one of the last `ncol`, set x label.
+        # Otherwise don't show tick labels.
         if (len(bands) - axnum - 1) < ncol:
             if toff == 0.:
                 bottomax.set_xlabel('time')
             else:
                 bottomax.set_xlabel('time - {0:.2f}'.format(toff))
-            #for l in bottomax.get_xticklabels():
-            #    l.set_rotation(22.5)
         else:
             for l in bottomax.get_xticklabels():
                 l.set_visible(False)
@@ -390,200 +474,3 @@ def plot_lc(data=None, model=None, bands=None, zp=25., zpsys='ab', pulls=True,
         return fig
     plt.savefig(fname, **kwargs)
     plt.close()
-
-def animate_source(source, label=None, fps=30, length=20.,
-                   phase_range=(None, None), wave_range=(None, None),
-                   match_peakphase=True, match_peakflux=True,
-                   peakwave=4000., fname=None, still=False):
-    """Animate spectral timeseries of model(s) using matplotlib.animation.
-
-    *Note:* Requires matplotlib v1.1 or higher.
-
-    Parameters
-    ----------
-    source : `~sncosmo.Source` or str or iterable thereof
-        The Source to animate or list of sources to animate.
-    label : str or list of str, optional
-        If given, label(s) for Sources, to be displayed in a legend on
-        the animation.
-    fps : int, optional
-        Frames per second. Default is 30.
-    length : float, optional
-        Movie length in seconds. Default is 15.
-    phase_range : (float, float), optional
-        Phase range to plot (in the timeframe of the first source if multiple
-        sources are given). `None` indicates to use the maximum extent of the
-        source(s).
-    wave_range : (float, float), optional
-        Wavelength range to plot. `None` indicates to use the maximum extent
-        of the source(s).
-    match_peakflux : bool, optional
-        For multiple sources, scale fluxes so that the peak of the spectrum
-        at the peak matches that of the first source. Default is
-        True.
-    match_peakphase : bool, optional
-        For multiple sources, shift additional sources so that the source's
-        reference phase matches that of the first source.
-    peakwave : float, optional
-        Wavelength used in match_peakflux and match_peakphase. Default is
-        4000.
-    fname : str, optional
-        If not `None`, save animation to file `fname`. Requires ffmpeg
-        to be installed with the appropriate codecs: If `fname` has
-        the extension '.mp4' the libx264 codec is used. If the
-        extension is '.webm' the VP8 codec is used. Otherwise, the
-        'mpeg4' codec is used. The first frame is also written to a
-        png.
-    still : bool, optional
-        When writing to a file, also save the first frame as a png file.
-        This is useful for displaying videos on a webpage.
-
-    Returns
-    -------
-    ani : `~matplotlib.animation.FuncAnimation`
-        Animation object that can be shown or saved.
-
-    Examples
-    --------
-    Compare the salt2 and hsiao sources:
-
-    >>> import matplotlib.pyplot as plt
-    >>> ani = animate_source(['salt2', 'hsiao'],  phase_range=(None, 30.),
-    ...                      wave_range=(2000., 9200.))  # doctest: +SKIP
-    >>> plt.show()  # doctest: +SKIP
-
-    Compare the salt2 source with ``x1=1`` to the same source with ``x1=0.``:
-
-    >>> m1 = sncosmo.get_source('salt2')  # doctest: +SKIP
-    >>> m1.set(x1=1.)                     # doctest: +SKIP
-    >>> m2 = sncosmo.get_source('salt2')  # doctest: +SKIP
-    >>> m2.set(x1=0.)                     # doctest: +SKIP
-    >>> ani = animate_source([m1, m2], label=['salt2, x1=1', 'salt2, x1=0'])
-    ... # doctest: +SKIP
-    >>> plt.show()                        # doctest: +SKIP
-    """
-    from matplotlib import animation
-
-    # Convert input to a list (if it isn't already).
-    if (not isiterable(source) or
-        isinstance(source, basestring)):
-        sources = [source]
-    else:
-        sources = source
-
-    # Check that all entries are Source or strings.
-    for m in sources:
-        if not (isinstance(m, basestring) or isinstance(m, Source)):
-            raise ValueError('str or Source instance expected for '
-                             'source(s)')
-    sources = [get_source(m) for m in sources]
-
-    # Get the source labels
-    if label is None:
-        labels = [None] * len(sources)
-    elif isinstance(label, basestring):
-        labels = [label]
-    else:
-        labels = label
-    if len(labels) != len(sources):
-        raise ValueError('if given, length of label must match '
-                         'that of source')
-
-    # Get a wavelength array for each source.
-    waves = [np.arange(m.minwave(), m.maxwave(), 10.) for m in sources]
-
-
-    # Phase offsets needed to match peak phases.
-    peakphases = [m.peakphase(peakwave) for m in sources]
-    if match_peakphase:
-        phase_offsets = [p - peakphases[0] for p in peakphases]
-    else:
-        phase_offsets =  [0.] * len(sources)
-
-    # Determine phase range to display.
-    minphase, maxphase = phase_range
-    if minphase is None:
-        minphase = min([sources[i].minphase() - phase_offsets[i] for
-                        i in range(len(sources))])
-    if maxphase is None:
-        maxphase = max([sources[i].maxphase() - phase_offsets[i] for
-                        i in range(len(sources))])
-    
-    # Determine the wavelength range to display.
-    minwave, maxwave = wave_range
-    if minwave is None:
-        minwave = min([m.minwave() for m in sources])
-    if maxwave is None:
-        maxwave = max([m.maxwave() for m in sources])
-
-    # source time interval between frames
-    phase_interval = (maxphase - minphase) / (length * fps)
-
-    # maximum flux density of entire spectrum at the peak phase
-    # for each source
-    max_fluxes = [np.max(m.flux(phase, w))
-                  for m, phase, w in zip(sources, peakphases, waves)]
-
-    # scaling factors
-    if match_peakflux:
-        peakfluxes = [m.flux(phase, peakwave)  # Not the same as max_fluxes!
-                      for m, phase in zip(sources, peakphases)]
-        scaling_factors = [peakfluxes[0] / f for f in peakfluxes]
-        global_max_flux = max_fluxes[0]
-    else:
-        scaling_factors = [1.] * len(sources)
-        global_max_flux = max(max_fluxes)
-
-    ymin = -0.06 * global_max_flux
-    ymax = 1.1 * global_max_flux
-
-    # Set up the figure, the axis, and the plot element we want to animate
-    fig = plt.figure()
-    ax = plt.axes(xlim=(minwave, maxwave), ylim=(ymin, ymax))
-    plt.axhline(y=0., c='k')
-    plt.xlabel('Wavelength ($\\AA$)') 
-    plt.ylabel('Flux Density ($F_\lambda$)')
-    phase_text = ax.text(0.05, 0.95, '', ha='left', va='top',
-                        transform=ax.transAxes)
-    empty_lists = 2 * len(sources) * [[]]
-    lines = ax.plot(*empty_lists, lw=1)
-    if label is not None:
-        for line, l in zip(lines, labels):
-            line.set_label(l)
-        legend = plt.legend(loc='upper right')
-
-    def init():
-        for line in lines:
-            line.set_data([], [])
-        phase_text.set_text('')
-        return tuple(lines) + (phase_text,)
-
-    def animate(i):
-        current_phase = minphase + phase_interval * i
-        for j in range(len(sources)):
-            y = sources[j].flux(current_phase + phase_offsets[j], waves[j])
-            lines[j].set_data(waves[j], y * scaling_factors[j])
-        phase_text.set_text('phase = {0:.1f}'.format(current_phase))
-        return tuple(lines) + (phase_text,)
-
-    ani = animation.FuncAnimation(fig, animate, init_func=init,
-                                  frames=int(fps*length), interval=(1000./fps),
-                                  blit=True)
-
-    # Save the animation as an mp4 or webm file.
-    # This requires that ffmpeg is installed.
-    if fname is not None:
-        if still:
-            i = fname.rfind('.')
-            stillfname = fname[:i] + '.png'
-            plt.savefig(stillfname) 
-        ext = fname[i+1:]
-        codec = {'mp4': 'libx264', 'webm': 'libvpx'}.get(ext, 'mpeg4')
-        ani.save(fname, fps=fps, codec=codec, extra_args=['-vcodec', codec],
-                 writer='ffmpeg_file', bitrate=1800)
-        plt.close()
-    else:
-        return ani
-
-# TODO This is for backwards compatibility only
-animate_model = animate_source
